/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * Portions of this software are based upon public domain software
 * originally written at the National Center for Supercomputing Applications,
 * University of Illinois, Urbana-Champaign.
 */

#include "httpd.h"
#include "ap_iol.h"
#include "iol_socket.h"
#include "apr_portable.h"
#include <malloc.h>

typedef struct {
    ap_iol iol;
    ap_socket_t *sock;
} iol_socket;

static ap_status_t win32_setopt(ap_iol *viol, ap_iol_option opt, const void *value)
{
    iol_socket *iol = (iol_socket *)viol;

    switch (opt) {
    case AP_IOL_TIMEOUT:
        ap_setsocketopt(iol->sock, APR_SO_TIMEOUT, 
                        *(const int*)value * (int)AP_USEC_PER_SEC);
	break;
    default:
	return APR_EINVAL;
    }
    return APR_SUCCESS;
}

static ap_status_t win32_getopt(ap_iol *viol, ap_iol_option opt, void *value)
{
    iol_socket *iol = (iol_socket *)viol;

    switch (opt) {
    case AP_IOL_TIMEOUT:
#if 0
	*(int *)value = iol->timeout; // ToDo: Impleent this
#endif
	break;
    default:
	return APR_EINVAL;
    }
    return APR_SUCCESS;
}
static ap_status_t win32_close(ap_iol *viol)
{
    iol_socket *iol = (iol_socket *)viol;
    return ap_close_socket(iol->sock);
}

static ap_status_t win32_write(ap_iol *viol, const char *buf, ap_size_t len, 
                               ap_ssize_t *nbytes)
{
    *nbytes = len;
    return ap_send(((iol_socket *)viol)->sock, buf, nbytes);
}
static ap_status_t win32_read( ap_iol *viol, char *buf, ap_size_t len, 
                               ap_ssize_t *nbytes)
{
    *nbytes = len;
    return ap_recv(((iol_socket *)viol)->sock, buf, nbytes);
}

static ap_status_t win32_writev(ap_iol *viol, const struct iovec *vec, int num, 
                                ap_ssize_t *nbytes)
{
    return ap_sendv(((iol_socket *)viol)->sock, vec, num, nbytes);
}

static ap_status_t win32_sendfile(ap_iol *viol, ap_file_t *file,
                                  ap_hdtr_t *hdtr, ap_off_t *offset, 
                                  ap_size_t *len,  ap_int32_t flags) 
{
    return ap_sendfile(((iol_socket *)viol)->sock, file, hdtr, offset, len, flags);
}

static ap_status_t win32_shutdown(ap_iol *viol, int how)
{
    iol_socket *iol = (iol_socket *)viol;

    return ap_shutdown(iol->sock, how);
}
    
static const ap_iol_methods socket_methods = {
    win32_close,
    win32_write,
    win32_writev,
    win32_read,
    win32_setopt,
    win32_getopt,
    win32_sendfile,
    win32_shutdown
};

ap_iol *win32_attach_socket(ap_pool_t *p, ap_socket_t *sock)
{
    iol_socket *iol;

//    iol = malloc(sizeof(iol_socket));
    iol = ap_palloc(p, sizeof(iol_socket));
    if (!iol)
        return (ap_iol*) NULL;
    iol->iol.methods = &socket_methods;
    iol->sock = sock;
    return (ap_iol *)iol;
}
